/*****************************************************************************
 * command.c: Locally sending commands to MCR user program
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Rishi Bhardwaj, <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/
#include <unistd.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <strings.h>

#include "command.h"

void printUsage(char **argv)
{
     printf("Usage: %s  -m metric_no.\n", argv[0]);
     printf("or otherwise: %s  -b beta_value  or  -a alpha or -f freeze\n", argv[0]);
     printf("metric_no. choses a metric type. All metric nos. are defined in mcr_route.h\n");
     printf("beta and alpha are constants used in routing metric\n");
}

int main(int argc, char**argv)
{
 //    unsigned int res=0, choice, time, ioctl_sock;
     int metric = -1;
     float beta = -1, alpha = -1;
     int sendfd;
     struct sockaddr_in sendSocket;
     struct command cmd;
     int freeze = -1, debug =-1;
     while (1) {
	  int opt;
	
	  opt = getopt(argc, argv, "b:m:a:f:d:");
	
	  if (opt == EOF)
	       break;

	  switch (opt) {
	  case 0:
	       break;

	  case 'm':
	       metric = atoi(optarg);
	       if (metric <= 0) {
		    printUsage(argv);
		    exit(1);
	       }
	       break;
	  case 'b':
	       beta =  atof(optarg);
	       break;
	  case 'a':
	       alpha = atof(optarg);
	       break;
	  case 'f':
	       freeze = atoi(optarg);
	       break;
	  case 'd':
	       debug = atoi(optarg);
	      break; 
	  }
	  
     }
	  
     printf("beta = %f , metric = %d , alpha = %f freeze = %d debug=%d\n", beta, metric, alpha, freeze, debug);
     if((beta == -1) && (metric == -1)&&(alpha == -1) &&(freeze == -1) && (debug == -1))
     {
	  printUsage(argv);
	  printf("\nNo option specified ?\n");
     } 
     else if(((beta !=-1) && (metric != -1)) ||((beta!= -1) && (alpha !=-1)) ||((alpha!=-1) && (metric !=-1)))
     {
	  printUsage(argv);
	  printf("\nOnly one option accepted\n");
     }
     else
     {
	  if(freeze != -1)
	  {
	       if(freeze == 1)
		    cmd.option = FREEZE_METRICS;
	       else
		    cmd.option = UNFREEZE_METRICS;

	  }
	  else if(debug != -1)
	  {
		if(debug == 0)
		     cmd.option = SUPPRESS_DEBUG_MSG ;
		else
		     cmd.option = START_DEBUG_MSG;
				 
	  }
	  else if(metric != -1)
	  {
	       cmd.option = CHANGE_METRIC;
	       cmd.field.metric = metric;
	  }
	  else if(beta !=-1)
	  {
	       cmd.option = CHANGE_BETA;
	       cmd.field.beta = beta;
	  }
	  else if(alpha != -1)
	  {
	       cmd.option = CHANGE_ALPHA;
	       cmd.field.alpha = alpha;
	  }
	  
	  sendfd = socket(AF_INET, SOCK_DGRAM, 0);
	  if(sendfd == -1)
	  {
	       //error
	       printf("\nsocket() error\n");
	       perror("socket");
	       exit(1);
	  }
	  bzero(&sendSocket, sizeof(struct sockaddr_in));
	  sendSocket.sin_family = AF_INET;
	  sendSocket.sin_addr.s_addr = htonl(INADDR_ANY);
	  sendSocket.sin_port = htons(COMMAND_LISTEN_PORT);
	  if(sendto(sendfd, (void*)&cmd,sizeof(cmd),0, (struct sockaddr*)&sendSocket, sizeof(sendSocket)) == -1)
	  {
	       perror("send");
	       exit(1);
	  }
	  close(sendfd);
     }

    return 0;
}

