/*****************************************************************************
 * integrated.h: Main header file
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 * Modifications by: Rishi Bhardwaj <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/

#ifndef _INTEGRATED_H
#define _INTEGRATED_H

/************  Default constants ***************************/

#define HELLO_PORT 9924                 // ALL userspace daemons should be compiled with the same port number

#define MAX_MESSAGE_LENGTH 1500  // maximum hello message that can every be sent (bytes)

#define STATUS_TIMEOUT_VALUE 600 // in seconds, time period of writing out status information


/************ Support for debugging **********************/

// data structure used for debugging
typedef struct struct_connectivity_table {
     struct in_addr addr;
     struct struct_connectivity_table *next;
} DebugTable;

typedef enum {
     NO_DEBUG = 0,
     LOCAL_ONLY = 1, // No interaction with bonding code, can be tested on single machine
     PARTIAL_LOCAL = 2 // Full Hello protocol, but partial routing protocol
} DEBUG_MODE_TYPE;

/************* Protocol packet structures ******************/

// Different types of protocol packets
typedef enum {
     LINK_HELLO_PKT,
     ROUTE_PKT,
     GATEWAY_PKT
} MCR_PACKET_TYPE;

// Main header of protocol packets
typedef struct struct_mcr_packet_header {
     int len;
     MCR_PACKET_TYPE type;

// myaddr can be moved out of the header in to 
// a separate option if a single node may have
// multiple addresses
     struct in_addr fromAddr; 
} McrPacketHeader;

/***************** Structure containing all userspace daemon data *****/

struct struct_timer;
struct struct_hello_server_data;
struct struct_routing_data;
struct gateway_data;

// Data maintained by server
typedef struct struct_server_data {
     unsigned int seq;                                   // seq number included in each protocol packet
     struct in_addr myIpAddr;     // node IP

     // for sending/receiving protocol packets
     struct sockaddr_in recvSocket;
     struct sockaddr_in sendBroadcastSocket;
     int sendfd;
     int receivefd;

     // RREPs, RERRs are unicasted
     struct sockaddr_in sendUnicastSocket;

     // for receving commands from another process/inter process communication
     int cmnd_recv_fd; 
     struct sockaddr_in cmnd_recv_sock;
     
     /* freeze_metrics is useful while conducting throughput experiments. 
      * if set to true, it freezes the node's fixed channel and the link quality metric of neighbors 
      * to the current values.
      */
     BOOL freeze_metrics;          
     
     // for communicating with kernel routing support module
     struct sockaddr_nl netlink_local;
     struct sockaddr_nl netlink_peer;
     int netlinkfd;

     // for communicating with kernel channel abstraction module
     int ioctlSock;
     struct ifreq ifr;

     // data structures for different protocol sub-components
     DebugTable *debugTable;
     struct struct_hello_server_data *helloData;
     struct struct_routing_data *routingData;
     struct gateway_data *gatewayData;
     struct struct_timer* timerQueue;
}ServerData;


/****************  Function declarations ******************/
void StatusTimeout(ServerData *serv);

#endif /* _INTEGRATED_H */
