/*****************************************************************************
 * mcr_gateway.h: Support for gateways
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 *
 *****************************************************************************/

#ifndef _MCR_GATEWAY_H
#define _MCR_GATEWAY_H

/********************* Default constants *********************************/

#define GATEWAY_BEACON_INTERVAL   2 // interval in seconds
#define GATEWAY_ENTRY_TICKS       3 // number of missed beacons before entry is removed

#define GATEWAY_MAX_TTL           7 // maximum distance gateway beacon can reach - ensures finite beacon time


/********************* Gateway beacon packet structure ********************/

typedef struct gateway_packet_header {
     struct in_addr addr;
     unsigned long seq;
     int ttl;
     int hops;
} GatewayPacketHeader;


/****************** Gateway table ************************/

typedef struct gateway_entry
{
     struct gateway_entry *next;
     struct in_addr addr; 
     unsigned long seq; // latest seq number heard from gateway
     int hops; // currently nearest gateway is used. this can be changed
     int ticks;
    
} GatewayEntry;


/*************** Main gateway protocol data ****************/

typedef struct gateway_data
{
     GatewayEntry *gatewayList;
     BOOL isGatewayNode; // Am I the gateway

     // If I am gateway
     unsigned long seq;
     struct in_addr internetGateway;
     BOOL gatewaySet;

     // If I am not gateway
     GatewayEntry *currentGateway;

} GatewayData;


/***********  Function declarations for external usage  *****************/

void GatewayInit(ServerData *serv);

void SetGatewayTimeout(ServerData *serv);

void GatewayTimeout(ServerData *serv);

void ProcessGatewayPacket(ServerData *serv, const McrPacketHeader *hdr, const char *dataPkt, int dataSize);

#endif /* _MCR_GATEWAY_H */
