/*****************************************************************************
 * mcr_ioctl.h: Support for communication with the kernel
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 * Modifications by: Rishi Bhardwaj <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/


#ifndef _MCR_IOCTL_H
#define _MCR_IOCTL_H

/****************** Ioctl definitions************************/
//Note: The Ioctl values have to be kept consistent with channel abstraction
// module code definitions in if_bonding.h. For ease of compilation, we did not
// directly include that file, and instead redefined the ioctl values

//#define SIOCDEVPRIVATE 0x89f0
#define BOND_MUIC_CHANNEL (SIOCDEVPRIVATE + 5)
#define BOND_MUIC_SWITCH_CHANNEL (SIOCDEVPRIVATE + 8)
#define BOND_MUIC_BROADCAST_CHANNEL (SIOCDEVPRIVATE + 7)
#define BOND_MUIC_ROUTE (SIOCDEVPRIVATE + 6)
#define BOND_MUIC_STATS  (SIOCDEVPRIVATE + 9) 
#define BOND_MUIC_CHANGE_CHAN_TIME (SIOCDEVPRIVATE + 10)

/************* Data structures for Ioctl calls *************************/
// These have to be kept consistent with the format expected by
// channel abstraction module - equivalent structures are in bond_muic.h
// Read the notes in bond_muic.h to see how to extend the code
// to support other interface capabilities, such as data rates.

struct muic_channel_req {
     int32_t             channel;   
     int             add;       
     char            slave[IFNAMSIZ];
};

struct muic_broadcast_req {
     int32_t         channel;       
     char            slave[IFNAMSIZ]; 
};

struct muic_unicast_req {
     uint32_t        addr;
     int32_t         channel;
     char            slave[IFNAMSIZ]; 
};

struct muic_channel_switch_req {
     int32_t         channel; 
     char            slave[IFNAMSIZ];
};

#define MUIC_MAX_CHANNELS 16
#define MAX_CHANNEL_USAGE 500        // in milliseconds, compensated for measurement loss
#define SWITCHING_COST    5000       // in microseconds

struct muic_stats_u_response
{
     int  nchan;
     int channel[MUIC_MAX_CHANNELS];
     unsigned int usage[MUIC_MAX_CHANNELS]; // in milliseconds
};                                                                                                                                          

/* Request for stats should mention the slave on which you expect the information */
struct muic_stats_u_req
{
     char slave[IFNAMSIZ];
     struct muic_stats_u_response *response;
};

/*Request for changing channel mininum/maximum time */
struct muic_chan_time_u_req
{
	char max_or_min ;           // 0 for changing chan minimum time, 1 for changing max time
	unsigned int chan_time;   // in milliseconds, valid values 10, 20, 30 ... 200
	
};


/********************  Function declarations for external usage  **********************/

// Netlink send
int Netlink_create_and_send_msg(ServerData *serv, int type, const struct in_addr *dest, const struct in_addr *nhop, u_int32_t lifetime, int flags);


// Interaction with kernel routing table
void Ioctl_AddRoute(ServerData *serv, struct in_addr *dst, struct in_addr *nextHop);

void Ioctl_DeleteRoute(ServerData *serv, struct in_addr *dst, struct in_addr *nextHop);

// Interaction with wireless card
void Ioctl_CardSetChannel(ServerData *serv, const char *interface, int channel);

// Interaction with 2.5 layer bonding code

void Ioctl_GetSwitchingStats(ServerData *serv, const char *interface, struct muic_stats_u_response *resp);

void Ioctl_AddValidChannel(ServerData *serv, const char *interface, int channel);

void Ioctl_SwitchChannel(ServerData *serv, const char *interface, int channel);

void Ioctl_AddUnicastEntry(ServerData *serv, const struct in_addr *addr, const char *interface, int channel);

void Ioctl_DeleteUnicastEntry(ServerData *serv, const struct in_addr *addr, const char *interface, int channel);

void Ioctl_AddBroadcastEntry(ServerData *serv, const char *interface, int channel);










#endif /* _MCR_IOCTL_H */
