/*****************************************************************************
 * timer.c: Support for timers
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 *
 *****************************************************************************/

#ifndef _TIMER_H
#define _TIMER_H

/************* Default constants ************************/

#define TIMER_GRANULARITY 10000   // in microseconds
#define MILLION 1000000                         // number of microseconds per second 

/********* Different kinds of supported timeouts *******/
typedef enum {
    HELLO_TIMEOUT = 1,
    ROUTING_TIMEOUT,
    STATUS_TIMEOUT,
    GATEWAY_TIMEOUT,
    SCAN_TIMEOUT
} TIMER_TYPE;

/*********** Structure of a timeout **********************/

typedef struct struct_timer {
    struct timeval time;
    TIMER_TYPE type;
    int subType;
    int timerData;
    struct struct_timer *next;
} Timer;


/***********  Function declarations for external usage  *****************/

void AddTimeout(ServerData *serv, struct timeval *delay, TIMER_TYPE type, int subType, int timerData);

struct timeval * GetNextTimeout(ServerData *serv);

void ExecuteTimeout(ServerData *serv);

#endif /* _TIMER_H */
