/*****************************************************************************
 * utilities.h: General purpose networking utilities
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 * Modifcations by: Rishi Bhardwaj, <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/

/* 
 * This file contains wrapper and utility functions for socket programming
 * The code here is based on the code in the book
 * Unix Network Programming: Volume 1, 2nd edition by Richard Stevens
 */

#ifndef _MCR_UTILITIES_H
#define _MCR_UTILITIES_H

#include <arpa/inet.h>
#include <errno.h>
#include <getopt.h>
#include <linux/if.h>
#include <linux/netlink.h>
#include <linux/wireless.h>
#include <net/route.h>
#include <netinet/in.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <syslog.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>


#define	SA struct sockaddr
#define MAXLINE 4096

// Macros for converting between structures and strings
#define        copyStructIntoPkt(structure, charPtr) \
            ( memcpy( (charPtr), &structure, sizeof(structure)) )          

#define        copyStructFromPkt(structure, charPtr) \
            ( memcpy(&structure, (charPtr), sizeof(structure)) ) 


// User-defined boolean type
typedef enum {
     FALSE = 0,
     TRUE
} BOOL;





/********************* Timer functions ******************************/

// compute t1 = t2 - t3, with assumption t2 > t3, if assumption fails, t1 is set to 0
void Gettimediff(struct timeval *t1, struct timeval *t2, struct timeval *t3);

void Gettimeofday(struct timeval *tv, struct timezone *tz);

BOOL IsTimeGreater(struct timeval *time1, struct timeval *time2);

/************************ Socket wrappers **************************/

void Bind(int fd, const struct sockaddr *sa, socklen_t salen);

ssize_t Recvfrom(int fd, void *ptr, size_t nbytes, int flags,
		 struct sockaddr *sa, socklen_t *salenptr);

void Sendto(int fd, const void *ptr, size_t nbytes, int flags,
	    const struct sockaddr *sa, socklen_t salen);

void Setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen);

int Socket(int family, int type, int protocol);

/*********************** Address translation ************************/

const char * Inet_ntop(int family, const void *addrptr, char *strptr, size_t len);

void Inet_pton(int family, const char *strptr, void *addrptr);

/******************** Error handling and logging ***********************/

void err_sys(const char *fmt, ...);

void err_quit(const char *fmt, ...);

void err_msg(const char *fmt, ...);

/******************** Debug functions, added by Rishi *****************/
static char inline *print_ip(__u32 addr)
{
     static char buf[16];

     sprintf(buf, "%d.%d.%d.%d",
	     0x0ff & addr,
	     0x0ff & (addr >> 8),
	     0x0ff & (addr >> 16), 0x0ff & (addr >> 24));

     return buf;
}

#endif /*_MCR_UTILITIES_H */ 
