/*****************************************************************************
 * kmcr_netlink.h: Support for communication with userspace
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 * Modifications by: Rishi Bhardwaj, <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/

#ifndef _KMCR_NETLINK_H
#define _KMCR_NETLINK_H

/************ The first part of this file is also used in compiling userspace daemon **********/

#define NETLINK_KMCR     10	// Type 10 and 12 were free with linux 2.4.26

#define MCR_GROUP_ID    2     // some dummy number to identify group

// Types of netlink messages  supported 
#define KMCR_BASE	       0x14

#define KMCR_ADDROUTE        (KMCR_BASE +1) // ask kernel to add a route

#define KMCR_DISCFAILED       (KMCR_BASE +2) // tell kernel no route found

#define KMCR_DELROUTE         (KMCR_BASE +3) // tell kernel to delete route

#define KMCR_ISROUTEVALID     (KMCR_BASE + 4) // ask kernel if a route is active

#define KMCR_ROUTEDISC        (KMCR_BASE +5) // tell user space to discover route

#define KMCR_ROUTEVALID       (KMCR_BASE + 6) // inform user space route is valid

#define KMCR_GATEWAY          (KMCR_BASE + 7) // tell kernel gateway addr - addr of 0 implies there is no gateway

#define KMCR_FWD_ERR	      (KMCR_BASE + 8) // inform user space, a packet could not be forwarded
// value used by userspace
#define TICKS_QUERY_KERNEL    2 // Number of seconds left, when kernel is called (measured before decrementing)

// netlink message structure
typedef struct kmcr_rt_msg {
     u_int32_t src;
     u_int32_t dest;
     u_int32_t nhop;
     u_int8_t flags;
     long route_expire; // in seconds
} kmcr_rt_msg_t;


/*************** Defintions used only while compiling kernel module *******************/
#ifdef __KERNEL__

int kmcr_netlink_init(void);

void kmcr_netlink_fini(void);

__u32 kmcr_get_gateway_addr(void);

void kmcr_netlink_send_rt_msg(int type, __u32 src, __u32 dest, unsigned long timeLeft);

#endif  /* __KERNEL__ */

#endif // _KMCR_NETLINK_H
