/*****************************************************************************
 * kmcr_table.h: Routing table support
 *
 * Copyright (C) 2006 University of Illinois
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * This code was written in the wireless networking group 
 * (http://www.crhc.uiuc.edu/wireless) at University of Illinois at Urbana-Champaign
 * Author: Pradeep Kyasanur, <kyasanur at crhc dot uiuc dot edu>
 * Modifications by: Rishi Bhardwaj, <rbhardw2 at uiuc dot edu>
 *
 *****************************************************************************/


#ifndef _KMCR_TABLE_H
#define _KMCR_TABLE_H

#include <linux/list.h>

// List of flags
#define ROUTE_DISC_PENDING    1
#define ROUTE_ACTIVE          2
#define ROUTE_EXPIRED         4

// Table data structure
struct route_entry 
{
     struct list_head ptr; // this has to be first element
     unsigned long last_access; // when route last used in jiffies
     unsigned long route_life; // how long after last access the route expires
     unsigned long pktCount; // how many packets forwarded for this route
     unsigned short flags;
     __u32 daddr;
     __u32 nhop;
};


// Exported functions
int kmcr_table_get(__u32 daddr, struct route_entry *ret);

int kmcr_table_add(__u32 daddr, __u32 nhop, unsigned long route_life,
		   unsigned short flags);

void kmcr_table_update(__u32 daddr, __u32 nhop, unsigned long route_life,
		       unsigned short flags);

int kmcr_table_access_update(__u32 daddr);

void kmcr_table_stats_update(__u32 daddr);

long kmcr_table_time_remaining(__u32 daddr);

int kmcr_table_del(__u32 daddr);

void kmcr_table_flush(void);

void kmcr_table_init(void);

void kmcr_table_fini(void);

#endif // _KMCR_TABLE_H
